using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.IO;
using System.Net;
using UnityEngine.Networking;

namespace hive.manager {
    public partial class HIVEManager {

        [System.Serializable]
        public class RootObjectForArrayRoot<T> {
            public T arr;
        }

        static IEnumerator requestGet<T>(string url, System.Action<T, System.Exception> callback) {
            var request =  UnityWebRequest.Get(url);
            request.chunkedTransfer = false;
            request.SetRequestHeader("Content-Type", "application/json;charset=utf-8");
            yield return request.SendWebRequest();
            do {  
                yield return null;  
            } while(!request.isDone);  
            try {
                string contents = request.downloadHandler.text;
                if (request.isHttpError || request.isNetworkError) {
                    //Debug.LogError(url + " : "+request.responseCode);
                    var errorMsg = "url : "+url+ "- code : " + request.responseCode + ", response : " + contents;
                    callback(default(T),new HttpListenerException((int)request.responseCode, errorMsg));
                } else {
                    // if (string.IsNullOrEmpty(contents)) {
                    //     callback(default(T), new System.NullReferenceException("Http Response is Empty"));
                    // }
                    contents = contents.Trim();//공백제거
                    if (contents.StartsWith("[")) { //Array Root Object라면
                        //Unity의 Json Uiilty가 Object Root만 파싱할수있기때문에 다음과 같이 처리.
                        callback(JsonUtility.FromJson<RootObjectForArrayRoot<T>>("{\"arr\":"+contents+"}").arr,null);
                    } else {
                        callback(JsonUtility.FromJson<T>(contents),null);
                    }
                }
            } catch(System.Exception e){
                callback(default(T),e);
            }
            yield return null;
        }

        #region VERSION API
            
        static string versionsAPI = baseUrl+"/versions";
        
        //Get /versions/available
        static void requestVersions(System.Action<List<AvailableDownload>,System.Exception> callback) {
            EditorCoroutine.start(requestGet<List<AvailableDownload>>(versionsAPI+"/available",callback));
        }

        //Get /versions/{current_version}/to/{update_version}
        static void requestVersionPatchData(string currentVersion, string targetVersion,System.Action<VersionMetadata, System.Exception> callback) {
            EditorCoroutine.start(requestGet<VersionMetadata>(versionsAPI+"/"+currentVersion+"/to/"+targetVersion,callback));
        }

        static void requestVersionPatchData(string targetVersion,System.Action<VersionMetadata, System.Exception> callback) {
            EditorCoroutine.start(requestGet<VersionMetadata>(versionsAPI+"/"+targetVersion,callback));
        }

        //Get /versions/hash/{hash}
        static void requestGetVersion(string hash, System.Action<VersionInfoFromHash, System.Exception> callback) {
            EditorCoroutine.start(requestGet<VersionInfoFromHash>(versionsAPI + "/hash/" + hash,callback));
        }
        #endregion

        #region NOTICES API
        static string noticesAPI = baseUrl+"/notices";
        //Get /version/{version_name}/description
        static void requestNotices(System.Action<List<AvailableNotice>,System.Exception> callback) {
            EditorCoroutine.start(requestGet<List<AvailableNotice>>(noticesAPI+"/available",callback));
        }
        static void requestNoticesAll(System.Action<List<AvailableNotice>,System.Exception> callback) {
            EditorCoroutine.start(requestGet<List<AvailableNotice>>(noticesAPI+"/all",callback));
        }
        
        #endregion
    }
}