/**
 * @file    HiveEditorLoader.cs
 * 
 * @author  disker
 * Copyright 2022 Com2uS Platform Corp.
 * @defgroup Hive.Unity.Editor
 * @{
 * @brief HIVE SDK 프로젝트 설정 지원 <br/><br/>
 */

namespace Hive.Unity.Editor
{
    using UnityEngine;
    using UnityEditor;
    using System.IO;
    
    [InitializeOnLoad]
    public class HiveEditorLoader
    {
        static HiveEditorLoader()
        {
            /*
                to support unity 2021: Deprecate "Assets/Plugins/Android/res", "Assets/Plugins/Android/assets" (v4.15.6)
                https://jira.com2us.com/jira/browse/GCPSDK4-678
            */
#if UNITY_2021_1_OR_NEWER
            string legacyAndroidResPath = "Assets/Plugins/Android/res";
            string androidResPath = "Assets/HiveSDK/hive.androidlib/src/main/res";
            if(Directory.Exists(legacyAndroidResPath)) {
                if(Directory.Exists(androidResPath)) {
                    Debug.Log("[HiveEditorLoader.cs] DeleteAsset \"" + legacyAndroidResPath + "\"");
                    AssetDatabase.DeleteAsset(legacyAndroidResPath);
                } else {
                    Debug.Log("[HiveEditorLoader.cs] Move from \"" + legacyAndroidResPath + "\" to \"" + androidResPath + "\"");
                    AssetDatabase.MoveAsset(legacyAndroidResPath, androidResPath);
                }
            }

            string legacyAndroidAssetsPath = "Assets/Plugins/Android/assets";
            string androidAssetsPath = "Assets/HiveSDK/hive.androidlib/src/main/assets";
            if(Directory.Exists(legacyAndroidAssetsPath)) {
                if(Directory.Exists(androidAssetsPath)) {
                    Debug.Log("[HiveEditorLoader.cs] DeleteAsset \"" + legacyAndroidAssetsPath + "\"");
                    AssetDatabase.DeleteAsset(legacyAndroidAssetsPath);
                } else {
                    Debug.Log("[HiveEditorLoader.cs] Move from \"" + legacyAndroidAssetsPath + "\" to \"" + androidAssetsPath + "\"");
                    AssetDatabase.MoveAsset(legacyAndroidAssetsPath, androidAssetsPath);
                }
            }
#endif
        }
    }
}
