/**
 * @file    HivePreprocess.cs
 * 
 * @author  disker
 * @date    2019-2022
 * @copyright	Copyright © Com2uS Platform Corporation. All Right Reserved.
 * @defgroup UnityEditor.HiveEditor
 * @{
 * @brief PrePrcessing on BuildTime <br/><br/>
 */

namespace UnityEditor.HiveEditor
{
    using UnityEditor;
    using UnityEditor.Build;
    using Hive.Unity.Editor;
    using UnityEngine;
    using System.IO;

#if UNITY_IOS
    using UnityEditor.iOS;
    using UnityEditor.iOS.Xcode;
#endif

#if UNITY_2018_1_OR_NEWER
    using UnityEditor.Build.Reporting;
    public class HivePreProcess : IPreprocessBuildWithReport
    {
        public int callbackOrder { get { return 0; } }
        public void OnPreprocessBuild(BuildReport report)
        {
            if (HiveConfigEditor.isAutoValidation())
            {
                Utility.XmlValidator.getInstance().execute();
            }

        /*
            to support unity 2021: Deprecate "Assets/Plugins/Android/res", "Assets/Plugins/Android/assets" (v4.15.6)
            https://jira.com2us.com/jira/browse/GCPSDK4-678
        */
#if UNITY_2021_1_OR_NEWER
            string legacyAndroidResPath = "Assets/Plugins/Android/res";
            string androidResPath = "Assets/HiveSDK/hive.androidlib/src/main/res";
            if(Directory.Exists(legacyAndroidResPath)) {
                if(Directory.Exists(androidResPath)) {
                    Debug.Log("[HivePreProcess.cs] DeleteAsset \"" + legacyAndroidResPath + "\"");
                    AssetDatabase.DeleteAsset(legacyAndroidResPath);
                } else {
                    Debug.Log("[HivePreProcess.cs] Move from \"" + legacyAndroidResPath + "\" to \"" + androidResPath + "\"");
                    AssetDatabase.MoveAsset(legacyAndroidResPath, androidResPath);
                }
            }

            string legacyAndroidAssetsPath = "Assets/Plugins/Android/assets";
            string androidAssetsPath = "Assets/HiveSDK/hive.androidlib/src/main/assets";
            if(Directory.Exists(legacyAndroidAssetsPath)) {
                if(Directory.Exists(androidAssetsPath)) {
                    Debug.Log("[HivePreProcess.cs] DeleteAsset \"" + legacyAndroidAssetsPath + "\"");
                    AssetDatabase.DeleteAsset(legacyAndroidAssetsPath);
                } else {
                    Debug.Log("[HivePreProcess.cs] Move from \"" + legacyAndroidAssetsPath + "\" to \"" + androidAssetsPath + "\"");
                    AssetDatabase.MoveAsset(legacyAndroidAssetsPath, androidAssetsPath);
                }
            }
#endif

        }
    }
#endif

//  IPreprocessBuild is Obsolete on (ver.2018.1~)
#if !UNITY_2018_1_OR_NEWER
    public class HivePreProcess : IPreprocessBuild
    {
        public int callbackOrder { get { return 0; } }
        public void OnPreprocessBuild(BuildTarget target, string path)
        {
            if (HiveConfigEditor.isAutoValidation())
            {
                Utility.XmlValidator.getInstance().execute();
            }

        }
    }
#endif

}
